#!/bin/bash

round() {
	if [ $(echo "$1 >= 0"| bc -l) -eq 1 ]; then
		echo $(printf %.$2f $(echo "scale=$2;(((10^$2)*$1)+0.5)/(10^$2)" | bc))
	else
		echo $(printf %.$2f $(echo "scale=$2;(((10^$2)*$1)-0.5)/(10^$2)" | bc))
	fi
};

abs() {
	echo $(echo "define abs(i) { if (i < 0) return (-i); return (i) } abs($1)" | bc)

};

probTable=$(cat << EOF
0;3;0.50
4;10;0.51
11;17;0.52
18;25;0.53
26;32;0.54
33;39;0.55
40;46;0.56
47;53;0.57
54;61;0.58
62;68;0.59
69;76;0.60
77;83;0.61
84;91;0.62
92;98;0.63
99;106;0.64
107;113;0.65
114;121;0.66
122;129;0.67
130;137;0.68
138;145;0.69
146;153;0.70
154;162;0.71
163;170;0.72
171;179;0.73
180;188;0.74
189;197;0.75
198;206;0.76
207;215;0.77
216;225;0.78
226;235;0.79
236;245;0.80
246;256;0.81
257;267;0.82
268;278;0.83
279;290;0.84
291;302;0.85
303;315;0.86
316;328;0.87
329;344;0.88
345;357;0.89
358;374;0.90
375;391;0.91
392;411;0.92
412;432;0.93
433;456;0.94
457;484;0.95
485;517;0.96
518;559;0.97
560;619;0.98
620;735;0.99
735;3000;1.00
EOF
)

while [[ $# -gt 0 ]] ; do
	case "$1" in 
		-b|--birthyear)	birthYear=$2;;
		-d|--dwz) dwzOld=$2	;;
		-c|--resultcount) dwzCalculated=$2;;
		-o|--opponentDwz) dwzList=$2;;
		-r|--results)results=$2;;
		*) echo "unknown option"; exit 1;;
	esac
	if [ -z $2 ]; then echo "Error, no empty inputs allowed!"; exit 1; fi
	shift 2;
done

re="^[[:digit:]]{4}$"
if [[ ! $birthYear =~ $re ]]; then echo "The birth year should contain 4 digits!"; exit 1; fi

dwzRe="^[[:digit:]]{3,4}|0$"
if [[ ! $dwzOld =~ $dwzRe ]]; then echo "The current dwz should contain 3 or 4 digits or a zero!"; exit 1; fi

dwzCalcRe="^[[:digit:]]{1,4}$"
if [[ ! $dwzCalculated =~ $dwzCalcRe ]]; then echo "The Count of former dwz evaluations should contain 1 to 4 digits!"; exit 1; fi

dwzListRe="^[[:digit:]]{3,4}(,[[:digit:]]{3,4})*$"
if [[ ! $dwzList =~ $dwzListRe ]]; then echo "The -o value should contain a comma separated list of dwz (3 to 4 digits) or one single dwz!"; exit 1; fi

resultsRe="^(0\.5|0|1)(,(0\.5|0|1))*$"
if [[ ! $results =~ $resultsRe ]]; then echo "The -r value should contain a comma separated list of chess results (0.5 or 0 or 1)!"; exit 1; fi

w=$(echo $results | tr ',' '+' | bc -l)
wFormatted=$(echo $w | sed 's/^\.5$/0,5/g;s/\.5/,5/;s/\.0//')

if [ ! $dwzOld -eq 0 ] && [ ! $dwzCalculated -eq 0 ] ; then 
	thisYear=$(date +%Y)
	if [ $birthYear -gt $thisYear ]; then
		echo "date of birth should be in the past!"; exit 1;
	fi
	age=$((thisYear - birthYear))
	if [ $age -le 20 ]; then j=5; fi
	if [ $age -gt 20 ] && [ $age -le 25 ] ; then j=10; fi
	if [ $age -gt 25 ]; then j=15; fi

	E0=$(echo "($dwzOld/1000)^4 + $j" | bc -l)
	a=1
	B=0
	IFS=,
	n=0
	w0=0
	for dwzOpp in $dwzList; do
		dwzDiff=$(echo "define abs(i) { if (i < 0) return (-i); return (i) } abs($dwzOld-$dwzOpp)" | bc)
		n=$((n + 1))
		while read probLine; do
			min=$(echo "$probLine" | awk -F ";" '{print $1}')
			max=$(echo "$probLine" | awk -F ";" '{print $2}')
			probability=$(echo "$probLine" | awk -F ";" '{print $3}')
			if [ $dwzDiff -ge $min ] && [ $dwzDiff -le $max ] ; then 
				if [ $dwzOld -lt $dwzOpp ]; then 
					probability="(1 - $probability)" 
				fi
				if [ $n -eq 1 ]; then
					w0String="$probability"
				else
					w0String="$w0String + $probability"
				fi
				break;
			fi
		done <<- ENDOF 
		$probTable
		ENDOF
		w0=$(echo "$w0String" | bc -l)
	done
	unset IFS

	if [ $age -lt 20 ] && [ $(echo "$w > $w0" | bc) -eq 1 ]; then
		a=$(echo "$dwzOld/2000" | bc -l)
		if [ $(echo "$a > 1" | bc -l) -eq 1 ]; then a=1; fi
		if [ $(echo "$a < 0.5" | bc -l) -eq 1 ]; then a=0.5; fi
	fi

	if [ $dwzOld -lt 1300 ] && [ $(echo "$w <= $w0" | bc -l) -eq 1 ]; then
		B=$(echo "e((1300-$dwzOld)/150)-1" | bc -l) 
	fi

	E=$(echo "$a*$E0 + $B" | bc -l)

	#echo "w=$w"
	#echo "w0=$w0"
	#echo "n=$n"
	#echo "age=$age"
	#echo "E0=$E0"
	#echo "j=$j"
	#echo "a=$a"
	#echo "B=$B"
	#echo "E=$E"

	if [ $(echo "$B == 0" | bc -l ) -eq 1 ]; then
		eMax=30
		resCount=$((5*$dwzCalculated))
		if [ $resCount -lt $eMax ]; then eMax=$resCount; fi
	else
		eMax=150
	fi

	if [ $(echo "$E < 5" | bc -l) -eq 1 ]; then
		E=5
	fi
	if [ $(echo "$E > $eMax" | bc -l ) -eq 1 ]; then
		E=$eMax
	fi

	#echo "EMax=$eMax"
	#echo "E-Change=$E"

	E=$(round $E 0)
	#echo "E-Round=$E"

	dwzChange=$(echo "800 * ($w - $w0) / ( $E + $n)" | bc -l )
	newDwz=$(echo "$dwzOld + $dwzChange" | bc -l )
	newDwzRound=$(round $newDwz 0)
	dwzChangeRound=$(round $dwzChange 0)

	#echo "dwzChange=$dwzChange"
	#echo "newDwz=$newDwz"
	#echo "newDwzRound=$newDwzRound"
	if [ $(echo "$dwzChangeRound > 0" | bc) -eq 1 ]; then
		dwzChangeRound="+$dwzChangeRound"
	fi
	echo "$wFormatted/$n;;$dwzChangeRound;;$newDwzRound"
else
	n=0
	IFS=,
	for dwzOpp in $dwzList; do
		n=$((n + 1))
	done
	unset IFS
	if [ $n -ge 5 ]; then
		if [ $(echo "($w != 0) && ($w != $n)" | bc -l) -eq 1 ]; then
			w=$(echo $results | tr ',' '+' | bc -l)
			if [ $(echo "$w >= ($n/2)" | bc -l) -eq 1 ]; then
				winPercent=$(echo "scale=2;$w/$n" | bc -l )
			else 
				winPercent=$(echo "scale=2;1-($w/$n)" | bc -l)
			fi
			dwzOppCumul=$(echo $dwzList | tr ',' '+' | bc -l)
			dwzOppMean=$(echo "$dwzOppCumul / $n" | bc -l)
			while read probLine; do
				min=$(echo "$probLine" | awk -F ";" '{print $1}')
				max=$(echo "$probLine" | awk -F ";" '{print $2}')
				probability=$(echo "$probLine" | awk -F ";" '{print $3}')
				if [ $(echo "$probability == $winPercent" | bc -l ) -eq 1 ]; then
					break;
				fi
			done <<- EOF
			$probTable
			EOF
			dwzDiffMean=$(echo "($max + $min)/ 2" | bc -l)
			if [ $(echo "$w >= ($n/2)" | bc -l) -eq 1 ]; then
				firstDwz=$(round $(echo "$dwzOppMean + $dwzDiffMean" | bc -l) 0)
			else 
				firstDwz=$(round $(echo "$dwzOppMean - $dwzDiffMean" | bc -l) 0)
			fi
			echo "$wFormatted/$n;;;;$firstDwz"
		else
			echo "$wFormatted/$n;;;;"
		fi
	else 
		echo "$wFormatted/$n;;;;"
	fi
fi
